﻿using System.Collections.ObjectModel;
using AsReader251GMAUIDemo.Platforms.Droid;
using static AsReader251GMAUIDemo.Platforms.Droid.AsReader251GDefine;

namespace AsReader251GMAUIDemo;

public partial class BarcodeScanPage : ContentPage
{
    private AsReader251GManager formManager;
    private AsReader251GDelegate asReader251GDelegate;
    private ObservableCollection<ItemData> BarcodeItems = new ObservableCollection<ItemData>();
    public ObservableCollection<ItemData> ItemData { get { return BarcodeItems; } }
    private Dictionary<string, ItemData> dataDic = new Dictionary<string, ItemData>();

    private AsReader251G_ActionState actionState;
    int count;

    public BarcodeScanPage()
    {
        InitializeComponent();
        count = 0;
        actionState = AsReader251G_ActionState.Stop;
        formManager = AsReader251GManager.sharedInstance();
        asReader251GDelegate = AsReader251GDelegate.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        BarcodeListView.ItemsSource = BarcodeItems;
        asReader251GDelegate.OnStateChanged = (AsReader251G_ConnectionState state) =>
        {
            OnStateChanged(state);
        };
        asReader251GDelegate.OnReadBarcode = (string p1, string p2, string p3) =>
        {
            OnReadBarcode(p1, p2, p3);
        };
        asReader251GDelegate.OnKeyEvent = (AsReader251G_KeyType p1, AsReader251G_KeyState p2) =>
        {
            OnKeyEvent(p1, p2);
        };
        asReader251GDelegate.OnExceptionOccurred = (int p1) =>
        {
            OnExceptionOccurred(p1);
        };
        asReader251GDelegate.OnActionChanged = (AsReader251G_ActionState p1) =>
        {
            OnActionChanged(p1);
        };
        try
        {
            formManager.SetBarcodeMode(true);
        }
        catch (Exception)
        {
        }
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        formManager.StopDecode();
        formManager.SetBarcodeMode(false);
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReader251G_ActionState.StartDecode)
        {
            formManager.StopDecode();
        }
        else if (actionState == AsReader251G_ActionState.Stop)
        {
            formManager.StartDecode();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        BarcodeItems.Clear();
    }
    void ASCIIButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Black;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Gray;
        formManager.AsReaderCharset = AsReader251G_Charset.ASCii;
    }
    void UTF8Button_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Black;
        ShiftJISButton.TextColor = Colors.Gray;
        formManager.AsReaderCharset = AsReader251G_Charset.UTF8;
    }
    void ShiftJISButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Black;
        formManager.AsReaderCharset = AsReader251G_Charset.ShiftJIS;
    }
    private void OnStateChanged(AsReader251G_ConnectionState state)
    {
        if (state != AsReader251G_ConnectionState.Connected)
        {
            MainThread.BeginInvokeOnMainThread(() =>
            {
                Navigation.PopAsync();
            });
        }
    }
    private void OnReadBarcode(string p1, string p2, string p3)
    {
        ItemData InventoryDataChanged;
        if (dataDic.ContainsKey(p1 + p3))
        {
            InventoryDataChanged = dataDic[p1 + p3];
            InventoryDataChanged.Count++;
            dataDic[p1 + p3] = InventoryDataChanged;
        }
        else
        {
            int nowCount = dataDic.Count;
            InventoryDataChanged = new ItemData(p1, p3, 1);
            dataDic.Add(p1 + p3, InventoryDataChanged);
            BarcodeItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            BarcodeListView.ItemsSource = null;
            BarcodeListView.ItemsSource = BarcodeItems;
            CountLabel.Text = count.ToString();
        });
    }
    private void OnKeyEvent(AsReader251G_KeyType p1, AsReader251G_KeyState p2)
    {
        if (p1 == AsReader251G_KeyType.Trigger)
        {
            if (p2 == AsReader251G_KeyState.KeyDown)
            {
                formManager.StartDecode();
            }
            else
            {
                formManager.StopDecode();
            }
        }
    }
    private void OnExceptionOccurred(int p1)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            DisplayAlert("Error", p1.ToString(), "Confirm");
        });
    }
    private void OnActionChanged(AsReader251G_ActionState p1)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (p1 == AsReader251G_ActionState.StartDecode)
            {
                actionState = AsReader251G_ActionState.StartDecode;
                StartScanButton.Text = "Stop Scan";
            }
            else if (p1 == AsReader251G_ActionState.Stop)
            {
                actionState = AsReader251G_ActionState.Stop;
                StartScanButton.Text = "Start Scan";
            }
        });

    }
}
